@@ -26,6 +26,7 @@
  */
 
 #include <unistd.h>
+#include <stdatomic.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -56,11 +57,26 @@ set_domain_gtt(int fd, uint32_t handle)
 }
 
 static void *
-mmap_bo(int fd, uint32_t handle)
+mmap_bo(int fd, uint32_t handle, uint64_t size)
 {
 	void *ptr;
 
-	ptr = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
+	ptr = gem_mmap__gtt(fd, handle, size, PROT_READ | PROT_WRITE);
+
+	return ptr;
+}
+
+static void *
+create_pointer_size(int fd, uint64_t size)
+{
+	uint32_t handle;
+	void *ptr;
+
+	handle = gem_create(fd, size);
+
+	ptr = mmap_bo(fd, handle, size);
+
+	gem_close(fd, handle);
 
 	return ptr;
 }
@@ -68,16 +84,7 @@ mmap_bo(int fd, uint32_t handle)
 static void *
 create_pointer(int fd)
 {
-	uint32_t handle;
-	void *ptr;
-
-	handle = gem_create(fd, OBJECT_SIZE);
-
-	ptr = mmap_bo(fd, handle);
-
-	gem_close(fd, handle);
-
-	return ptr;
+	return create_pointer_size(fd, OBJECT_SIZE);
 }
 
 static void
@@ -115,6 +122,20 @@ test_access(int fd)
 			  MAP_SHARED, fd2, mmap_arg.offset));
 }
 
+static void
+test_zero_extend(int i915)
+{
+	struct drm_i915_gem_mmap_gtt arg = {};
+	uint64_t redzone[16];
+
+	memset(redzone, 0xc5, sizeof(redzone));
+	arg.handle = gem_create(i915, 4096);
+
+	igt_assert_eq(ioctl(i915, DRM_IOCTL_I915_GEM_MMAP_GTT, &arg), 0);
+
+	gem_close(i915, arg.handle);
+}
+
 static void
 test_short(int fd)
 {
@@ -322,6 +343,141 @@ test_pf_nonblock(int i915)
 	igt_spin_free(i915, spin);
 }
 
+static void
+test_isolation(int i915)
+{
+	struct drm_i915_gem_mmap_gtt mmap_arg;
+	int A = gem_reopen_driver(i915);
+	int B = gem_reopen_driver(i915);
+	uint64_t offset_a, offset_b;
+	uint32_t a, b;
+	void *ptr;
+
+	a = gem_create(A, 4096);
+	b = gem_open(B, gem_flink(A, a));
+
+	mmap_arg.handle = a;
+	do_ioctl(A, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg);
+	offset_a = mmap_arg.offset;
+
+	mmap_arg.handle = b;
+	do_ioctl(B, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg);
+	offset_b = mmap_arg.offset;
+
+	igt_info("A: {fd:%d, handle:%d, offset:%"PRIx64"}\n",
+		 A, a, offset_a);
+	igt_info("B: {fd:%d, handle:%d, offset:%"PRIx64"}\n",
+		 B, b, offset_b);
+
+	close(B);
+
+	ptr = mmap64(0, 4096, PROT_READ, MAP_SHARED, A, offset_a);
+	igt_assert(ptr != MAP_FAILED);
+	munmap(ptr, 4096);
+
+	close(A);
+
+	ptr = mmap64(0, 4096, PROT_READ, MAP_SHARED, A, offset_a);
+	igt_assert(ptr == MAP_FAILED);
+}
+
+static void
+test_close_race(int i915)
+{
+	const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
+	_Atomic uint32_t *handles;
+
+	handles = mmap64(0, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	igt_assert(handles != MAP_FAILED);
+
+	igt_fork(child, ncpus + 1) {
+		do {
+			struct drm_i915_gem_mmap_gtt mmap_arg = {};
+			const int i = 1 + random() % ncpus;
+			uint32_t old;
+
+			mmap_arg.handle = gem_create(i915, 4096);
+			old = atomic_exchange(&handles[i], mmap_arg.handle);
+			ioctl(i915, DRM_IOCTL_GEM_CLOSE, &old);
+
+			if (ioctl(i915,
+				  DRM_IOCTL_I915_GEM_MMAP_GTT,
+				  &mmap_arg) != -1) {
+				void *ptr;
+
+				ptr = mmap64(0, 4096,
+					     PROT_WRITE, MAP_SHARED, i915,
+					     mmap_arg.offset);
+				if (ptr != MAP_FAILED) {
+					*(volatile uint32_t *)ptr = 0;
+					munmap(ptr, 4096);
+				}
+			}
+		} while (!READ_ONCE(handles[0]));
+	}
+
+	sleep(20);
+	handles[0] = 1;
+	igt_waitchildren();
+
+	for (int i = 1; i <= ncpus; i++)
+		ioctl(i915, DRM_IOCTL_GEM_CLOSE, handles[i]);
+	munmap(handles, 4096);
+}
+
+static void
+test_flink_race(int i915)
+{
+	const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
+	_Atomic uint32_t *handles;
+
+	handles = mmap64(0, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	igt_assert(handles != MAP_FAILED);
+
+	igt_fork(child, ncpus + 1) {
+		int fd = gem_reopen_driver(i915);
+
+		do {
+			struct drm_i915_gem_mmap_gtt mmap_arg = {};
+			const int i = 1 + random() % ncpus;
+			uint32_t old;
+
+			old = atomic_exchange(&handles[i],
+					      gem_create(i915, 4096));
+			if (!old)
+				continue;
+
+			mmap_arg.handle =
+				gem_open(fd, gem_flink(i915, old));
+			gem_close(i915, old);
+
+			if (ioctl(fd,
+				  DRM_IOCTL_I915_GEM_MMAP_GTT,
+				  &mmap_arg) != -1) {
+				void *ptr;
+
+				ptr = mmap64(0, 4096,
+					     PROT_WRITE, MAP_SHARED, fd,
+					     mmap_arg.offset);
+				if (ptr != MAP_FAILED) {
+					*(volatile uint32_t *)ptr = 0;
+					munmap(ptr, 4096);
+				}
+			}
+
+			gem_close(fd, mmap_arg.handle);
+		} while (!READ_ONCE(handles[0]));
+	}
+
+	sleep(20);
+	handles[0] = 1;
+	igt_waitchildren();
+
+	for (int i = 1; i <= ncpus; i++)
+		ioctl(i915, DRM_IOCTL_GEM_CLOSE, handles[i]);
+	munmap(handles, 4096);
+}
+
 static void
 test_write_gtt(int fd)
 {
@@ -332,7 +488,7 @@ test_write_gtt(int fd)
 	dst = gem_create(fd, OBJECT_SIZE);
 
 	/* prefault object into gtt */
-	dst_gtt = mmap_bo(fd, dst);
+	dst_gtt = mmap_bo(fd, dst, OBJECT_SIZE);
 	set_domain_gtt(fd, dst);
 	memset(dst_gtt, 0, OBJECT_SIZE);
 	munmap(dst_gtt, OBJECT_SIZE);
@@ -466,6 +622,9 @@ test_hang(int fd)
 
 				gtt[0][x] = patterns[next_pattern];
 				gtt[1][x] = patterns[next_pattern];
+
+				if (READ_ONCE(control->done))
+					break;
 			}
 
 			last_pattern = next_pattern;
@@ -656,9 +815,6 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b, int ncpus)
 	uint64_t huge_object_size, i;
 	unsigned mode = CHECK_RAM;
 
-	igt_fail_on_f(intel_gen(devid) >= 11 && ncpus > 1,
-		      "Please adjust your expectations, https://bugs.freedesktop.org/show_bug.cgi?id=110882\n");
-
 	switch (huge) {
 	case -2:
 		huge_object_size = gem_mappable_aperture_size() / 4;
@@ -822,10 +978,16 @@ thread_fault_concurrent(void *closure)
 	int n;
 
 	for (n = 0; n < 32; n++) {
+		unsigned int id = (n + t->id) % 32;
+		uint32_t sz = *t->ptr[id] - 1;
+		int idx = rand() % sz + 1;
+
 		if (n & 1)
-			*t->ptr[(n + t->id) % 32] = val;
+			t->ptr[id][idx] = val;
 		else
-			val = *t->ptr[(n + t->id) % 32];
+			val = t->ptr[id][idx];
+
+		val++;
 	}
 
 	return NULL;
@@ -839,7 +1001,10 @@ test_fault_concurrent(int fd)
 	int n;
 
 	for (n = 0; n < 32; n++) {
-		ptr[n] = create_pointer(fd);
+		uint32_t sz = (n + 1) << 19; /* 512KiB increments */
+
+		ptr[n] = create_pointer_size(fd, sz);
+		*ptr[n] = sz / sizeof(uint32_t); /* num_elems for convenience */
 	}
 
 	for (n = 0; n < 64; n++) {
@@ -848,21 +1013,13 @@ test_fault_concurrent(int fd)
 		pthread_create(&thread[n].thread, NULL, thread_fault_concurrent, &thread[n]);
 	}
 
+	sleep(2);
+
 	for (n = 0; n < 64; n++)
 		pthread_join(thread[n].thread, NULL);
 
-	for (n = 0; n < 32; n++) {
-		munmap(ptr[n], OBJECT_SIZE);
-	}
-}
-
-static void
-run_without_prefault(int fd,
-			void (*func)(int fd))
-{
-	igt_disable_prefault();
-	func(fd);
-	igt_enable_prefault();
+	for (n = 0; n < 32; n++)
+		munmap(ptr[n], *ptr[n] * sizeof(uint32_t));
 }
 
 static int mmap_ioctl(int i915, struct drm_i915_gem_mmap_gtt *arg)
@@ -883,9 +1040,17 @@ igt_main
 	if (igt_run_in_simulation())
 		OBJECT_SIZE = 1 * 1024 * 1024;
 
-	igt_fixture
+	igt_fixture {
 		fd = drm_open_driver(DRIVER_INTEL);
 
+		/*
+		 * If the HW lacks or does not expose an aperture for indirect
+		 * detiling access from untrusted userspace to the objects,
+		 * the kernel does an early rejection of the mmap_gtt ioctl.
+		 */
+		gem_require_mappable_ggtt(fd);
+	}
+
 	igt_subtest("bad-object") {
 		uint32_t real_handle = gem_create(fd, 4096);
 		uint32_t handles[20];
@@ -935,16 +1100,18 @@ igt_main
 		test_read_write2(fd, READ_AFTER_WRITE);
 	igt_subtest("fault-concurrent")
 		test_fault_concurrent(fd);
-	igt_subtest("basic-read-no-prefault")
-		run_without_prefault(fd, test_read);
-	igt_subtest("basic-write-no-prefault")
-		run_without_prefault(fd, test_write);
-	igt_subtest("basic-write-gtt-no-prefault")
-		run_without_prefault(fd, test_write_gtt);
 	igt_subtest("basic-write-cpu-read-gtt")
 		test_write_cpu_read_gtt(fd);
 	igt_subtest("basic-wc")
 		test_wc(fd);
+	igt_subtest("isolation")
+		test_isolation(fd);
+	igt_subtest("zero-extend")
+		test_zero_extend(fd);
+	igt_subtest("close-race")
+		test_close_race(fd);
+	igt_subtest("flink-race")
+		test_flink_race(fd);
 	igt_subtest("pf-nonblock")
 		test_pf_nonblock(fd);
 
@@ -996,17 +1163,34 @@ igt_main
 			for (const struct copy_mode *m = copy_modes; m->suffix; m++) {
 				igt_subtest_f("%s-copy%s", s->prefix, m->suffix)
 					test_huge_copy(fd,
-							s->size,
-							m->tiling_x,
-							m->tiling_y,
-							1);
+						       s->size,
+						       m->tiling_x,
+						       m->tiling_y,
+						       1);
+
+				igt_subtest_f("cpuset-%s-copy%s", s->prefix, m->suffix) {
+					cpu_set_t cpu, old;
+
+					sched_getaffinity(0, sizeof(old), &old);
+
+					CPU_ZERO(&cpu);
+					CPU_SET(0, &cpu);
+					igt_assert(sched_setaffinity(0, sizeof(cpu), &cpu) == 0);
+					test_huge_copy(fd,
+						       s->size,
+						       m->tiling_x,
+						       m->tiling_y,
+						       2);
+
+					igt_assert(sched_setaffinity(0, sizeof(old), &old) == 0);
+				}
 
 				igt_subtest_f("forked-%s-copy%s", s->prefix, m->suffix)
 					test_huge_copy(fd,
-							s->size,
-							m->tiling_x,
-							m->tiling_y,
-							ncpus);
+						       s->size,
+						       m->tiling_x,
+						       m->tiling_y,
+						       ncpus);
 			}
 	}
 
